'use strict'; // express-xmlrpc - xml-rpc middleware for express
// xml-rpc spec: http://www.xmlrpc.com/spec

var _regeneratorRuntime = require("@babel/runtime-corejs2/regenerator");

var _JSON$stringify = require("@babel/runtime-corejs2/core-js/json/stringify");

var _asyncToGenerator = require("@babel/runtime-corejs2/helpers/asyncToGenerator");

var Deserializer = require('./lib/deserializer.js');

var Serializer = require('./lib/serializer.js');

var Client = require('./lib/client.js');

var debug = require('debug')('express-xmlrpc:index'); // for serializing xmlrpc responses inside api methods


exports.serializeResponse = Serializer.serializeMethodResponse; // (params)

exports.serializeFault = Serializer.serializeFault; // (code, msg)

exports.sendResult = function (result, req, res, next) {
  if (req.body.multicall) {
    return result;
  }

  res.send(exports.serializeResponse(result));
}; // client to make xmlrpc method calls


exports.createClient = function (options) {
  return new Client(options, false);
}; // middleware to parse body of xmlrpc method call & add to request
// * method -> req.body.method
// * parameters -> req.body.params


exports.bodyParser = function (req, res, next) {
  var deserializer = new Deserializer();
  deserializer.deserializeMethodCall(req, function (error, method, params) {
    if (error !== null) {
      req.body = null;
      console.error('failed to deserialize method call from body:', error);
      next();
    } else {
      req.body = {
        method: method,
        params: params
      };
      next();
    }
  });
}; // generate route handler for xmlrpc method reqests from api & context
// api is an object with method names mapped to handler functions:
// { methodName: methodNameHandler[(request, response, next)], .. }
// context is an optional context object to be mapped to this inside of call


exports.apiHandler = function (api, context, onError, onMiss) {
  return (
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(req, res, next) {
        var results, calls, i, method, callResult, _method;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // if xml wasnt successfully parsed respond with fault
                if (!req.body) {
                  res.send(exports.serializeFault(-32700, 'parse error: not well formed'));
                } // handle system.multicall requests


                if (!(req.body.method === 'system.multicall')) {
                  _context.next = 28;
                  break;
                }

                req.body.multicall = true;
                results = [];
                _context.prev = 4;
                calls = req.body.params[0]; // calls are passed as list

                i = 0;

              case 7:
                if (!(i < calls.length)) {
                  _context.next = 19;
                  break;
                }

                req.body.method = calls[i]['methodName'];
                req.body.params = calls[i]['params'];
                debug("calling multicall method: '".concat(req.body.method) + "(".concat(_JSON$stringify(req.body.params), ")'"));
                method = api[req.body.method];
                _context.next = 14;
                return method.call(context, req);

              case 14:
                callResult = _context.sent;
                results.push([callResult]);

              case 16:
                i++;
                _context.next = 7;
                break;

              case 19:
                res.send(exports.serializeResponse(results));
                _context.next = 26;
                break;

              case 22:
                _context.prev = 22;
                _context.t0 = _context["catch"](4);
                req.body.method = 'system.multicall'; // put multicall back for error
                // if error handler was given call it with error, req & res

                if (onError) {
                  onError.call(context, _context.t0, req, res, next); // otherwise log & send generic xmlrpc fault
                } else {
                  console.error("error calling method '".concat(req.body.method, ":"), _context.t0);
                  res.send(exports.serializeFault(-32500, "error calling method '".concat(req.body.method, "'")));
                  next(_context.t0);
                }

              case 26:
                _context.next = 42;
                break;

              case 28:
                if (!(req.body.method in api)) {
                  _context.next = 41;
                  break;
                }

                debug("calling method '".concat(req.body.method) + "(".concat(_JSON$stringify(req.body.params), ")'"));
                _context.prev = 30;
                _method = api[req.body.method]; // rejected promises wont be caught unless they are awaited

                _context.next = 34;
                return _method.call(context, req, res, next);

              case 34:
                _context.next = 39;
                break;

              case 36:
                _context.prev = 36;
                _context.t1 = _context["catch"](30);

                // if error handler was given call it with error, req & res
                if (onError) {
                  onError.call(context, _context.t1, req, res, next); // otherwise log & send generic xmlrpc fault
                } else {
                  console.error("error calling method '".concat(req.body.method, ":"), _context.t1);
                  res.send(exports.serializeFault(-32500, "error calling method '".concat(req.body.method, "'")));
                  next(_context.t1);
                }

              case 39:
                _context.next = 42;
                break;

              case 41:
                // if miss handler was given call it
                if (onMiss) {
                  onMiss.call(context, req, res, next); // otherwise send generic xmlrpc fault
                } else {
                  res.send(exports.serializeFault(-32601, "requested method '".concat(req.body.method, "' not found")));
                }

              case 42:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[4, 22], [30, 36]]);
      }));

      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }()
  );
};